package org.xbib.event.async;

import org.xbib.event.async.streams.ReadStream;

/**
 * A timeout stream is triggered by a timer, the {@link Handler} will be call when the timer is fired,
 * it can be once or several times depending on the nature of the timer related to this stream. The
 * {@link ReadStream#endHandler(Handler)} will be called after the timer handler has been called.
 * <p>
 * Pausing the timer inhibits the timer shots until the stream is resumed. Setting a null handler callback cancels
 * the timer.
 *
 */
public interface TimeoutStream extends ReadStream<Long> {

  @Override
  TimeoutStream exceptionHandler(Handler<Throwable> handler);

  @Override
  TimeoutStream handler(Handler<Long> handler);

  @Override
  TimeoutStream pause();

  @Override
  TimeoutStream resume();

  @Override
  TimeoutStream fetch(long amount);

  @Override
  TimeoutStream endHandler(Handler<Void> endHandler);

  /**
   * Cancels the timeout. Note this has the same effect as calling {@link #handler(Handler)} with a null
   * argument.
   */
  void cancel();

}
