package org.xbib.event.async.spi;

import org.xbib.event.async.impl.AsyncBuilder;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ThreadFactory;

/**
 * The interface for a factory used to obtain an external
 * {@code ExecutorService}.
 */
public interface ExecutorServiceFactory extends AsyncServiceProvider {

  /**
   * Default instance that delegates to {@link Executors#newFixedThreadPool(int, ThreadFactory)}
   */
  ExecutorServiceFactory INSTANCE = (threadFactory, concurrency, maxConcurrency) ->
    Executors.newFixedThreadPool(maxConcurrency, threadFactory);

  @Override
  default void init(AsyncBuilder builder) {
    if (builder.executorServiceFactory() == null) {
      builder.executorServiceFactory(this);
    }
  }

  /**
   * Create an ExecutorService
   *
   * @param threadFactory  A {@link ThreadFactory} which must be used by the
   *                       created {@link ExecutorService} to create threads. Null
   *                       indicates there is no requirement to use a specific
   *                       factory.
   * @param concurrency    The target level of concurrency or 0 which indicates
   *                       unspecified
   * @param maxConcurrency A hard limit to the level of concurrency required,
   *                       should be greater than {@code concurrency} or 0 which
   *                       indicates unspecified.
   *
   * @return an {@link ExecutorService} that can be used to run tasks
   */
  ExecutorService createExecutor(ThreadFactory threadFactory, Integer concurrency, Integer maxConcurrency);

}
