package org.xbib.event.async.yield;

import java.util.concurrent.CompletableFuture;
import java.util.function.BiConsumer;

/**
 * Asynchronous traversal.
 * We use a traverse method as its first choice to
 * implement AsyncQuery operations.
 * This is a special kind of traversal that disallows individually access.
 */
public interface AsyncTraverser<T> {
    /**
     * Yields elements sequentially until all elements have been
     * processed or an exception is thrown.
     * The given consumer is invoked with the result (or null if none)
     * and the exception (or null if none).
     *
     * @return A CompletableFuture to signal finish to enable cancellation
     * through its cancel() method.
     */
    CompletableFuture<Void> subscribe(BiConsumer<? super T, ? super Throwable> cons);
}
