package org.xbib.event.async.yield.ops;

import org.xbib.event.async.yield.Advancer;
import org.xbib.event.async.yield.Query;
import org.xbib.event.async.yield.Traverser;
import org.xbib.event.async.yield.Yield;

import java.util.function.Function;

public class FlatMap<T, R> implements Advancer<R>, Traverser<R> {

    private final Query<T> upstream;

    private final Function<? super T, ? extends Query<? extends R>> mapper;

    private Query<? extends R> src;

    public FlatMap(Query<T> query, Function<? super T, ? extends Query<? extends R>> mapper) {
        this.upstream = query;
        this.mapper = mapper;
        src = new Query<>(Advancer.empty(), Traverser.empty());
    }

    @Override
    public void traverse(Yield<? super R> yield) {
        upstream.traverse(elem ->
                mapper.apply(elem).traverse(yield));

    }

    @Override
    public boolean tryAdvance(Yield<? super R> yield) {
        while (!src.tryAdvance(yield)) {
            if (!upstream.tryAdvance(t -> src = mapper.apply(t)))
                return false;
        }
        return true;
    }
}
