package org.xbib.event.async.yield.primitives.lng.ops;

import org.xbib.event.async.yield.Yield;
import org.xbib.event.async.yield.primitives.lng.LongAdvancer;
import org.xbib.event.async.yield.primitives.lng.LongQuery;
import org.xbib.event.async.yield.primitives.lng.LongTraverser;
import org.xbib.event.async.yield.primitives.lng.LongYield;

import java.util.function.LongPredicate;

public class LongTakeWhile implements LongAdvancer, LongTraverser {
    private final LongQuery upstream;
    private final LongPredicate predicate;
    private boolean hasNext;

    public LongTakeWhile(LongQuery upstream, LongPredicate predicate) {
        this.upstream = upstream;
        this.predicate = predicate;
        this.hasNext = true;
    }

    @Override
    public void traverse(LongYield yield) {
        upstream.shortCircuit(item -> {
            if (!predicate.test(item)) {
                Yield.bye();
            }
            yield.ret(item);
        });
    }

    @Override
    public boolean tryAdvance(LongYield yield) {
        if (!hasNext) return false; // Once predicate is false it finishes the iteration
        LongYield takeWhile = item -> {
            if (predicate.test(item)) {
                yield.ret(item);
            } else {
                hasNext = false;
            }
        };
        return upstream.tryAdvance(takeWhile) && hasNext;
    }
}
