package org.xbib.event.persistence;

import org.xbib.datastructures.json.tiny.Json;

import java.io.IOException;
import java.io.Writer;
import java.net.URI;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class FileJsonCodec implements Codec<Map<String, Object>> {

    private final URI uri;

    private final ReentrantReadWriteLock readWriteLock;

    public FileJsonCodec(URI uri) {
        this.uri = uri;
        this.readWriteLock = new ReentrantReadWriteLock();
    }

    @Override
    public Map<String, Object> read(String key) throws IOException {
        try {
            readWriteLock.readLock().lock();
            Path p = openOrCreate(key);
            return Files.exists(p) ? Json.toMap(Files.newBufferedReader(p)) : null;
        } finally {
            readWriteLock.readLock().unlock();
        }
    }

    @Override
    public void write(String key, Map<String, Object> data) throws IOException {
        try {
            readWriteLock.writeLock().lock();
            Path p = openOrCreate(key);
            try (Writer writer = Files.newBufferedWriter(p)) {
                writer.write(Json.toString(data));
            }
        } finally {
            readWriteLock.writeLock().unlock();
        }
    }

    private Path openOrCreate(String key) throws IOException {
        Path path = Paths.get(uri);
        Files.createDirectories(path);
        return path.resolve(key);
    }
}
