package org.xbib.event.wal;

/**
 * <p>
 * Provides a callback mechanism by which applicable listeners can be notified
 * when a WriteAheadRepository is synched (via the
 * {@link WriteAheadRepository#checkpoint()} method) or one of its partitions is
 * synched via
 * {@link WriteAheadRepository#update(java.util.Collection, boolean)} with a
 * value of <code>true</code> for the second argument.
 * </p>
 *
 * <p>
 * It is not required that an implementation of {@link WriteAheadRepository}
 * support this interface. Those that do generally will require that the
 * listener be injected via the constructor.
 * </p>
 *
 * <p>
 * All implementations of this interface must be thread-safe.
 * </p>
 *
 * <p>
 * The {@link #onSync(int)} method will always be called while the associated
 * partition is locked. The {@link #onGlobalSync()} will always be called while
 * the entire repository is locked.
 * </p>
 *
 */
public interface SyncListener {

    /**
     * This method is called whenever a specific partition is synched via the
     * {@link WriteAheadRepository#update(java.util.Collection, boolean)} method
     *
     * @param partitionIndex the index of the partition that was synched
     */
    void onSync(int partitionIndex);

    /**
     * This method is called whenever the entire
     * <code>WriteAheadRepository</code> is synched via the
     * {@link WriteAheadRepository#checkpoint()} method.
     */
    void onGlobalSync();

    SyncListener NOP_SYNC_LISTENER = new SyncListener() {
        @Override
        public void onSync(int partitionIndex) {
        }

        @Override
        public void onGlobalSync() {
        }
    };
}
