package org.xbib.event.bus;

import java.util.ArrayList;
import java.util.List;
import org.junit.jupiter.api.Test;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * Validate that {@link EventBus} behaves carefully when listeners publish their own events.
 */
public class ReentrantEventsTest {

    static final String FIRST = "one";
    static final Double SECOND = 2.0d;

    final EventBus bus = new EventBus();

    @Test
    public void testNoReentrantEvents() {
        ReentrantEventsHater hater = new ReentrantEventsHater();
        bus.register(hater);
        bus.post(FIRST);
        assertEquals(List.of(FIRST, SECOND), hater.eventsReceived, "ReentrantEventHater expected 2 events");
    }

    public class ReentrantEventsHater {
        boolean ready = true;
        List<Object> eventsReceived = new ArrayList<>();

        @Subscribe
        public void listenForStrings(String event) {
            eventsReceived.add(event);
            ready = false;
            try {
                bus.post(SECOND);
            } finally {
                ready = true;
            }
        }

        @Subscribe
        public void listenForDoubles(Double event) {
            assertTrue(ready, "I received an event when I wasn't ready!");
            eventsReceived.add(event);
        }
    }

    @Test
    public void testEventOrderingIsPredictable() {
        EventProcessor processor = new EventProcessor();
        bus.register(processor);
        EventRecorder recorder = new EventRecorder();
        bus.register(recorder);
        bus.post(FIRST);
        assertEquals(List.of(FIRST, SECOND), recorder.eventsReceived, "EventRecorder expected events in order");
    }

    class EventProcessor {
        @Subscribe
        public void listenForStrings(String event) {
            bus.post(SECOND);
        }
    }

    static class EventRecorder {
        List<Object> eventsReceived = new ArrayList<>();

        @Subscribe
        public void listenForEverything(Object event) {
            eventsReceived.add(event);
        }
    }
}
