package org.xbib.event.loop;

import org.xbib.event.loop.EventExecutor;

import java.util.Objects;

/**
 * The {@link CompleteFuture} which is failed already.  It is
 * recommended to use {@link EventExecutor#newFailedFuture(Throwable)}
 * instead of calling the constructor of this future.
 */
public final class FailedFuture<V> extends CompleteFuture<V> {

    private final Throwable cause;

    /**
     * Creates a new instance.
     *
     * @param executor the {@link EventExecutor} associated with this future
     * @param cause   the cause of failure
     */
    public FailedFuture(EventExecutor executor, Throwable cause) {
        super(executor);
        this.cause = Objects.requireNonNull(cause, "cause");
    }

    @Override
    public Throwable cause() {
        return cause;
    }

    @Override
    public boolean isSuccess() {
        return false;
    }

    @Override
    public Future<V> sync() {
        throw new RuntimeException(cause);
    }

    @Override
    public Future<V> syncUninterruptibly() {
        throw new RuntimeException(cause);
    }

    @Override
    public V getNow() {
        return null;
    }
}
