package org.xbib.event.syslog;

import java.net.InetAddress;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

public class DefaultSyslogMessage implements SyslogMessage {

    private final Builder builder;

    private DefaultSyslogMessage(Builder builder) {
        this.builder = builder;
    }

    public static Builder builder() {
        return new Builder();
    }

    @Override
    public LocalDateTime date() {
        return builder.date;
    }

    @Override
    public InetAddress remoteAddress() {
        return builder.remoteAddress;
    }

    @Override
    public String rawMessage() {
        return builder.rawMessage;
    }

    @Override
    public MessageType type() {
        return builder.type;
    }

    @Override
    public Integer level() {
        return builder.level;
    }

    @Override
    public Integer version() {
        return builder.version;
    }

    @Override
    public Integer facility() {
        return builder.facility;
    }

    @Override
    public String host() {
        return builder.host;
    }

    @Override
    public String message() {
        return builder.message;
    }

    @Override
    public String processId() {
        return builder.processId;
    }

    @Override
    public String tag() {
        return builder.tag;
    }

    @Override
    public String messageId() {
        return builder.messageId;
    }

    @Override
    public String appName() {
        return builder.appName;
    }

    @Override
    public List<org.xbib.datastructures.api.Builder> structuredData() {
        return builder.structuredData;
    }

    @Override
    public String deviceVendor() {
        return builder.deviceVendor;
    }

    @Override
    public String deviceProduct() {
        return builder.deviceProduct;
    }

    @Override
    public String deviceVersion() {
        return builder.deviceVersion;
    }

    @Override
    public String deviceEventClassId() {
        return builder.deviceEventClassId;
    }

    @Override
    public String name() {
        return builder.name;
    }

    @Override
    public String severity() {
        return builder.severity;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("date=").append(builder.date)
                .append(",remoteAddress=").append(builder.remoteAddress)
                .append(",rawMessage=").append(builder.rawMessage)
                .append(",type=").append(builder.type)
                .append(",level=").append(builder.level)
                .append(",version=").append(builder.version)
                .append(",facility=").append(builder.facility)
                .append(",host=").append(builder.host)
                .append(",message=").append(builder.message)
                .append(",processId=").append(builder.processId)
                .append(",tag=").append(builder.tag)
                .append(",messageId=").append(builder.messageId)
                .append(",appName=").append(builder.appName)
                .append(",structuredData=").append(builder.structuredData);
        return sb.toString();
    }

    public static class Builder implements SyslogMessage.Builder {

        LocalDateTime date;

        InetAddress remoteAddress;

        String rawMessage;

        MessageType type;

        Integer level;

        Integer version;

        Integer facility;

        String host;

        String message;

        String processId;

        String tag;

        String messageId;

        String appName;

        List<org.xbib.datastructures.api.Builder> structuredData;

        String deviceVendor;

        String deviceProduct;

        String deviceVersion;

        String deviceEventClassId;

        String name;

        String severity;

        Map<String, Object> map;

        private Builder() {
        }

        public Builder date(LocalDateTime date) {
            this.date = date;
            return this;
        }

        public Builder remoteAddress(InetAddress remoteAddress) {
            this.remoteAddress = remoteAddress;
            return this;
        }

        public Builder rawMessage(String rawMessage) {
            this.rawMessage = rawMessage;
            return this;
        }

        public Builder type(MessageType type) {
            this.type = type;
            return this;
        }

        public Builder level(Integer level) {
            this.level = level;
            return this;
        }

        public Builder version(Integer version) {
            this.version = version;
            return this;
        }

        public Builder facility(Integer facility) {
            this.facility = facility;
            return this;
        }

        public Builder host(String host) {
            this.host = host;
            return this;
        }

        public Builder message(String message) {
            this.message = message;
            return this;
        }

        public Builder processId(String processId) {
            this.processId = processId;
            return this;
        }

        public Builder tag(String tag) {
            this.tag = tag;
            return this;
        }

        public Builder messageId(String messageId) {
            this.messageId = messageId;
            return this;
        }

        public Builder appName(String appName) {
            this.appName = appName;
            return this;
        }

        public Builder structuredData(List<org.xbib.datastructures.api.Builder> structuredData) {
            this.structuredData = structuredData;
            return this;
        }

        public Builder deviceVendor(String deviceVendor) {
            this.deviceVendor = deviceVendor;
            return this;
        }

        public Builder deviceProduct(String deviceProduct) {
            this.deviceProduct = deviceProduct;
            return this;
        }

        public Builder deviceVersion(String deviceVersion) {
            this.deviceVersion = deviceVersion;
            return this;
        }

        public Builder deviceEventClassId(String deviceEventClassId) {
            this.deviceEventClassId = deviceEventClassId;
            return this;
        }

        public Builder name(String name) {
            this.name = name;
            return this;
        }

        public Builder severity(String severity) {
            this.severity = severity;
            return this;
        }

        public Builder map(Map<String, Object> map) {
            this.map = map;
            return this;
        }

        @Override
        public SyslogMessage build() {
            return new DefaultSyslogMessage(this);
        }
    }
}
