package org.xbib.event.syslog;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToMessageDecoder;

import java.net.InetSocketAddress;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.List;

@ChannelHandler.Sharable
public class TCPSyslogMessageDecoder extends MessageToMessageDecoder<ByteBuf> {

    private final Charset charset;

    public TCPSyslogMessageDecoder(Charset charset) {
        this.charset = charset;
    }

    public TCPSyslogMessageDecoder() {
        this(StandardCharsets.UTF_8);
    }

    @Override
    protected void decode(ChannelHandlerContext channelHandlerContext, ByteBuf byteBuf, List<Object> output) {
        final InetSocketAddress socketAddress = (InetSocketAddress) channelHandlerContext.channel().remoteAddress();
        final String rawMessage = byteBuf.toString(this.charset);
        output.add(DefaultSyslogRequest.builder()
                        .receivedDate(LocalDateTime.now())
                        .remoteAddress(socketAddress.getAddress())
                        .rawMessage(rawMessage)
                        .build()
        );
    }

}
