package org.xbib.event.syslog;

import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.codec.MessageToMessageDecoder;

import java.net.InetAddress;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.List;

@ChannelHandler.Sharable
public class UDPSyslogMessageDecoder extends MessageToMessageDecoder<DatagramPacket> {

    private final Charset charset;

    public UDPSyslogMessageDecoder(Charset charset) {
        this.charset = charset;
    }

    public UDPSyslogMessageDecoder() {
        this(StandardCharsets.UTF_8);
    }

    @Override
    protected void decode(ChannelHandlerContext channelHandlerContext,
                          DatagramPacket datagramPacket,
                          List<Object> output) {
        final String rawMessage = datagramPacket.content().toString(this.charset);
        final InetAddress inetAddress = datagramPacket.sender().getAddress();
        output.add(DefaultSyslogRequest.builder()
                        .receivedDate(LocalDateTime.now())
                        .rawMessage(rawMessage)
                        .remoteAddress(inetAddress)
                        .build()
        );
    }
}
